# -*- coding: utf-8 -*-

"""Jinja environment filters for Pynuts."""

from base64 import b64encode
from flask import escape
from wtforms import BooleanField, DateField, DateTimeField
from wtforms.ext.sqlalchemy.fields import (
    QuerySelectField, QuerySelectMultipleField)
from .fields import UploadField, ImageField


def data(field):
    """Field data beautifier.

    QuerySelectMultipleField
      Renders comma-separated data.
    QuerySelectField
      Renders the selected value.
    BooleanField
      Renders '✓' or '✕'
    DateField
      Renders the date
    ImageField
      Renders the image
    UploadField
      Renders a link to the uploaded file

    Example:

    .. sourcecode:: html+jinja

        <dd>{{ field | data }}</dd>

    """
    if isinstance(field, QuerySelectMultipleField):
        if field.data:
            return escape(
                u', '.join(field.get_label(data) for data in field.data))
        else:
            return u'∅'
    elif isinstance(field, QuerySelectField):
        if field.data:
            return escape(field.get_label(field.data))
    elif isinstance(field, BooleanField):
        return u'✓' if field.data else u'✕'
    elif isinstance(field, DateField) or isinstance(field, DateTimeField):
        if field.data:
            return field.data.strftime(field.format)
    elif isinstance(field, ImageField):
        if field.data:
            return u'<img src="%s" alt="" />' % (
                field.upload_set.url(field.data))
    elif isinstance(field, UploadField):
        if field.data:
            return u'<a href="%s">%s</a>' % (
                field.upload_set.url(field.data),
                field.data)
    return escape(field.data)


def base64(bytestring):
    """Return base64 bytes corresponding to the bytestring.

    Example:

    .. sourcecode:: html+jinja

        <img src="data:image/png;base64,{{ image | base64" }} />

    """
    return b64encode(bytestring)
